<?php

namespace Restaurant\Restaurant\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Restaurant\Restaurant\Http\Requests\RestaurantRequest;
use Restaurant\Restaurant\Interfaces\RestaurantRepositoryInterface;
use Restaurant\Restaurant\Models\Restaurant;

/**
 * Resource controller class for restaurant.
 */
class RestaurantResourceController extends BaseController
{

    /**
     * Initialize restaurant resource controller.
     *
     * @param type RestaurantRepositoryInterface $restaurant
     *
     * @return null
     */
    public function __construct(RestaurantRepositoryInterface $restaurant)
    {
        parent::__construct();
        $this->repository = $restaurant;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Restaurant\Restaurant\Repositories\Criteria\RestaurantResourceCriteria::class);
    }

    /**
     * Display a list of restaurant.
     *
     * @return Response
     */
    public function index(RestaurantRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Restaurant\Restaurant\Repositories\Presenter\RestaurantPresenter::class)
                ->$function();
        }

        $restaurants = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('restaurant::restaurant.names'))
            ->view('restaurant::restaurant.index', true)
            ->data(compact('restaurants', 'view'))
            ->output();
    }

    /**
     * Display restaurant.
     *
     * @param Request $request
     * @param Model   $restaurant
     *
     * @return Response
     */
    public function show(RestaurantRequest $request, Restaurant $restaurant)
    {

        if ($restaurant->exists) {
            $view = 'restaurant::restaurant.show';
        } else {
            $view = 'restaurant::restaurant.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('restaurant::restaurant.name'))
            ->data(compact('restaurant'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new restaurant.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RestaurantRequest $request)
    {

        $restaurant = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('restaurant::restaurant.name')) 
            ->view('restaurant::restaurant.create', true) 
            ->data(compact('restaurant'))
            ->output();
    }

    /**
     * Create new restaurant.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RestaurantRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $restaurant                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('restaurant::restaurant.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('restaurant/restaurant/' . $restaurant->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/restaurant/restaurant'))
                ->redirect();
        }

    }

    /**
     * Show restaurant for editing.
     *
     * @param Request $request
     * @param Model   $restaurant
     *
     * @return Response
     */
    public function edit(RestaurantRequest $request, Restaurant $restaurant)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('restaurant::restaurant.name'))
            ->view('restaurant::restaurant.edit', true)
            ->data(compact('restaurant'))
            ->output();
    }

    /**
     * Update the restaurant.
     *
     * @param Request $request
     * @param Model   $restaurant
     *
     * @return Response
     */
    public function update(RestaurantRequest $request, Restaurant $restaurant)
    {
        try {
            $attributes = $request->all();

            $restaurant->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('restaurant::restaurant.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('restaurant/restaurant/' . $restaurant->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('restaurant/restaurant/' . $restaurant->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the restaurant.
     *
     * @param Model   $restaurant
     *
     * @return Response
     */
    public function destroy(RestaurantRequest $request, Restaurant $restaurant)
    {
        try {

            $restaurant->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('restaurant::restaurant.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('restaurant/restaurant/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('restaurant/restaurant/' . $restaurant->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple restaurant.
     *
     * @param Model   $restaurant
     *
     * @return Response
     */
    public function delete(RestaurantRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('restaurant::restaurant.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('restaurant/restaurant'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/restaurant/restaurant'))
                ->redirect();
        }

    }

    /**
     * Restore deleted restaurants.
     *
     * @param Model   $restaurant
     *
     * @return Response
     */
    public function restore(RestaurantRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('restaurant::restaurant.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/restaurant/restaurant'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/restaurant/restaurant/'))
                ->redirect();
        }

    }

}
